@@ -1,6 +1,6 @@
 package Module::Build::Base;
 
-# $Id $
+# $Id: Base.pm,v 1.17 2002/03/31 12:43:06 ken Exp $
 
 use strict;
 use Config;
@@ -9,22 +9,53 @@ use File::Find ();
 use File::Path ();
 use File::Basename ();
 use File::Spec ();
+use Data::Dumper ();
 
 sub new {
   my $package = shift;
-  my $self = bless {
-		    # Don't save these defaults in config_dir.
-		    build_script => 'Build',
-		    config_dir => '_build',
-		    @_,
-		    new_cleanup => {},
-		   }, $package;
-  
-  my ($action, $args) = $self->cull_args(@ARGV);
-  die "Too early to specify a build action '$action'.  Do ./$self->{build_script} $action instead.\n"
+  my %input = @_;
+
+  my $args   = delete $input{args}   || {};
+  my $config = delete $input{config} || {};
+
+  my ($action, $cmd_args) = __PACKAGE__->cull_args(@ARGV);
+  die "Too early to specify a build action '$action'.  Do 'Build $action' instead.\n"
     if $action;
 
-  $self->{args} = {%Config, @_, %$args};
+  my $cmd_config;
+  if ($cmd_args->{config}) {
+    # XXX need to hashify this string better (deal with quoted whitespace)
+    $cmd_config->{$1} = $2 while $cmd_args->{config} =~ /(\w+)=(\S+)/;
+  } else {
+    $cmd_config = {};
+  }
+  delete $cmd_args->{config};
+
+  # Extract our 'properties' from $cmd_args, the rest are put in 'args'
+  my $cmd_properties = {};
+  foreach my $key (keys %$cmd_args) {
+    $cmd_properties->{$key} = delete $cmd_args->{$key} if __PACKAGE__->valid_property($key);
+  }
+
+  # 'args' are arbitrary user args.
+  # 'config' is Config.pm and its overridden values.
+  # 'properties' is stuff Module::Build needs in order to work.  They get saved in _build/.
+  # Anything else in $self doesn't get saved.
+
+  my $self = bless {
+		    args => {%$args, %$cmd_args},
+		    config => {%Config, %$config, %$cmd_config},
+		    properties => {
+				   build_script => 'Build',
+				   config_dir => '_build',
+				   prereq => {},
+				   recommended => {},
+				   PL_files => {},
+				   %input,
+				   %$cmd_properties,
+				  },
+		    new_cleanup => {},
+		   }, $package;
 
   $self->check_manifest;
   $self->check_prereq;
@@ -44,6 +75,27 @@ sub resume {
   return $self;
 }
 
+{
+  # XXX huge hack alert - will revisit this later
+  my %valid_properties = map {$_ => 1}
+    qw(
+       module_name
+       module_version
+       module_version_from
+       prereq
+       recommended
+       PL_files
+       config_dir
+       build_script
+       debugger
+       verbose
+       c_source
+       autosplit
+      );
+
+  sub valid_property { exists $valid_properties{$_[1]} }
+}
+
 # XXX Problem - if Module::Build is loaded from a different directory,
 # it'll look for (and perhaps destroy/create) a _build directory.
 sub subclass {
@@ -84,17 +136,17 @@ EOF
 
 sub find_version {
   my ($self) = @_;
-  return if exists $self->{args}{module_version};
+  return if exists $self->{properties}{module_version};
   
-  if (exists $self->{args}{module_version_from}) {
-    my $version = $self->version_from_file($self->{args}{module_version_from});
-    $self->{args}{module_version} = $version;
-    delete $self->{args}{module_version_from};
+  if (exists $self->{properties}{module_version_from}) {
+    my $version = $self->version_from_file($self->{properties}{module_version_from});
+    $self->{properties}{module_version} = $version;
+    delete $self->{properties}{module_version_from};
   } else {
     # Try to find the version in 'module_name'
-    my $chief_file = $self->module_name_to_file($self->{args}{module_name});
-    die "Can't find module '$self->{args}{module_name}' for version check" unless defined $chief_file;
-    $self->{args}{module_version} = $self->version_from_file($chief_file);
+    my $chief_file = $self->module_name_to_file($self->{properties}{module_name});
+    die "Can't find module '$self->{properties}{module_name}' for version check" unless defined $chief_file;
+    $self->{properties}{module_version} = $self->version_from_file($chief_file);
   }
 }
 
@@ -152,8 +204,8 @@ sub write_cleanup {
 }
 
 sub config_file {
-  my ($self) = @_;
-  return File::Spec->catfile($self->{config_dir}, $_[1]);
+  my $self = shift;
+  return File::Spec->catfile($self->{properties}{config_dir}, @_);
 }
 
 sub read_config {
@@ -161,14 +213,9 @@ sub read_config {
   
   my $file = $self->config_file('build_params');
   open my $fh, $file or die "Can't read '$file': $!";
-  
-  while (<$fh>) {
-    if (/^(\w+)$/) {
-      $self->{args}{$1} = undef;
-    } elsif (/^(\w+)=(.*)/) {
-      $self->{args}{$1} = $2;
-    }
-  }
+  my $ref = eval do {local $/; <$fh>};
+  die if $@;
+  ($self->{args}, $self->{config}, $self->{properties}) = @$ref;
   close $fh;
   
   my $cleanup_file = $self->config_file('cleanup');
@@ -184,36 +231,30 @@ sub read_config {
 sub write_config {
   my ($self) = @_;
   
-  File::Path::mkpath($self->{config_dir});
-  -d $self->{config_dir} or die "Can't mkdir $self->{config_dir}: $!";
+  File::Path::mkpath($self->{properties}{config_dir});
+  -d $self->{properties}{config_dir} or die "Can't mkdir $self->{properties}{config_dir}: $!";
   
   my $file = $self->config_file('build_params');
   open my $fh, ">$file" or die "Can't create '$file': $!";
-  
-  foreach my $key (sort keys %{$self->{args}} ) {
-    if (!defined $self->{args}{$key}) {
-      print $fh "$key\n";
-
-    } elsif ($self->{args}{$key} =~ /\n/) {
-      warn "Sorry, can't handle newlines in config data '$key' (yet)\n";
-
-    } else {
-      print $fh "$key=$self->{args}{$key}\n";
-    }
-  }
+  local $Data::Dumper::Terse = 1;
+  print $fh Data::Dumper::Dumper([$self->{args}, $self->{config}, $self->{properties}]);
+  close $fh;
 }
 
 sub check_prereq {
   my $self = shift;
-  return 1 unless $self->{prereq};
 
   my $pass = 1;
-  while (my ($modname, $spec) = each %{$self->{prereq}}) {
+  while (my ($modname, $spec) = each %{$self->{properties}{prereq}}) {
     my $thispass = $self->check_installed_version($modname, $spec);
     warn "WARNING: $@\n" unless $thispass;
     $pass &&= $thispass;
   }
 
+  while (my ($modname, $spec) = each %{$self->{properties}{recommended}}) {
+    warn "NOTE: $@\n" unless $self->check_installed_version($modname, $spec);
+  }
+
   if (!$pass) {
     warn "ERRORS FOUND IN PREREQUISITES.  You may wish to install the versions ".
          "of the modules indicated above before proceeding with this installation.\n";
@@ -221,6 +262,7 @@ sub check_prereq {
   return $pass;
 }
 
+# I wish I could set $!, but I can't.
 sub check_installed_version {
   my ($self, $modname, $spec) = @_;
 
@@ -240,13 +282,13 @@ sub check_installed_version {
   if ($spec =~ /^\s*([\w.]+)\s*$/) { # A plain number, maybe with dots, letters, and underscores
     @conditions = (">= $spec");
   } else {
-    @conditions = split /\s*,\s*/, $self->{prereq}{$modname};
+    @conditions = split /\s*,\s*/, $self->{properties}{prereq}{$modname};
   }
 
   foreach (@conditions) {
     if ($_ !~ /^\s*  (<=?|>=?|==|!=)  \s*  [\w.]+  \s*$/x) {
       $@ = "Invalid prerequisite condition for $modname: $_";
-      next;
+      return 0;
     }
     unless (eval "\$version $_") {
       $@ = "$modname version $version is installed, but we need version $_";
@@ -259,14 +301,14 @@ sub check_installed_version {
 
 sub rm_previous_build_script {
   my $self = shift;
-  if (-e $self->{build_script}) {
-    print "Removing previous file '$self->{build_script}'\n";
-    unlink $self->{build_script} or die "Couldn't remove '$self->{build_script}': $!";
+  if (-e $self->{properties}{build_script}) {
+    print "Removing previous file '$self->{properties}{build_script}'\n";
+    unlink $self->{properties}{build_script} or die "Couldn't remove '$self->{properties}{build_script}': $!";
   }
 }
 
 sub make_build_script_executable {
-  chmod 0544, $_[0]->{build_script};
+  chmod 0544, $_[0]->{properties}{build_script};
 }
 
 sub print_build_script {
@@ -275,7 +317,7 @@ sub print_build_script {
   my $build_package = ref($self);
 
   my ($config_dir, $build_script, $build_dir) = 
-    ($self->{config_dir}, $self->{build_script},
+    ($self->{properties}{config_dir}, $self->{properties}{build_script},
      File::Spec->rel2abs(File::Basename::dirname($0)));  # XXX should be property of $self
 
   my @myINC = @INC;
@@ -286,16 +328,19 @@ sub print_build_script {
   my $quoted_INC = join ', ', map "'$_'", @myINC;
 
   print $fh <<EOF;
-$self->{args}{startperl} -w
+$self->{config}{startperl} -w
 
 BEGIN { \@INC = ($quoted_INC) }
 
 chdir('$build_dir') or die 'Cannot chdir to $build_dir: '.\$!;
 use $build_package;
 
+# This should have just enough arguments to be able to bootstrap the rest.
 my \$build = resume $build_package (
-  config_dir => '$config_dir',
-  build_script => '$build_script',
+  properties => {
+    config_dir => '$config_dir',
+    build_script => '$build_script',
+  },
 );
 eval {\$build->dispatch};
 my \$err = \$@;
@@ -310,9 +355,9 @@ sub create_build_script {
   
   $self->rm_previous_build_script;
 
-  print("Creating new '$self->{build_script}' script for ",
-	"'$self->{args}{module_name}' version '$self->{args}{module_version}'\n");
-  open my $fh, ">$self->{build_script}" or die "Can't create '$self->{build_script}': $!";
+  print("Creating new '$self->{properties}{build_script}' script for ",
+	"'$self->{properties}{module_name}' version '$self->{properties}{module_version}'\n");
+  open my $fh, ">$self->{properties}{build_script}" or die "Can't create '$self->{properties}{build_script}': $!";
   $self->print_build_script($fh);
   close $fh;
   
@@ -342,19 +387,27 @@ sub check_manifest {
 sub dispatch {
   my $self = shift;
   
+  my (%p, $args, $action);
   if (@_) {
-    $self->{action} = shift;
-    $self->{args} = {%{$self->{args}}, @_};
+    ($action, %p) = @_;
+    $args = $p{args} ? delete($p{args}) : {};
   } else {
-    my ($action, $args) = $self->cull_args(@ARGV);
-    $self->{action} = $action || 'build';
-    $self->{args} = {%{$self->{args}}, %$args};
+    ($action, $args) = $self->cull_args(@ARGV);
+
+    # Extract our 'properties' from $args
+    foreach my $key (keys %$args) {
+      $p{$key} = delete $args->{$key} if __PACKAGE__->valid_property($key);
+    }
   }
 
-  my $action = "ACTION_$self->{action}";
-  print("No method '$action' defined.\n"), return unless $self->can($action);
+  $self->{action} = $action || 'build';
+  $self->{args} = {%{$self->{args}}, %$args};
+  $self->{properties} = {%{$self->{properties}}, %p};
+
+  my $method = "ACTION_$self->{action}";
+  print("No method '$method' defined.\n"), return unless $self->can($method);
   
-  return $self->$action;
+  return $self->$method;
 }
 
 sub cull_args {
@@ -423,9 +476,9 @@ sub ACTION_test {
   
   $self->depends_on('build');
   
-  local $Test::Harness::switches = '-w -d' if $self->{args}{debugger};
-  local $Test::Harness::verbose = $self->{args}{verbose} || 0;
-  local $ENV{TEST_VERBOSE} = $self->{args}{verbose} || 0;
+  local $Test::Harness::switches = '-w -d' if $self->{properties}{debugger};
+  local $Test::Harness::verbose = $self->{properties}{verbose} || 0;
+  local $ENV{TEST_VERBOSE} = $self->{properties}{verbose} || 0;
 
   # Make sure we test the module in blib/
   {
@@ -439,6 +492,8 @@ sub ACTION_test {
   push @tests, 'test.pl'                          if -e 'test.pl';
   push @tests, @{$self->rscan_dir('t', qr{\.t$})} if -e 't' and -d _;
   if (@tests) {
+    # Work around a Test::Harness bug that loses the particular perl we're running under
+    local $^X = $self->{config}{perlpath} unless $Test::Harness::VERSION gt '2.01';
     Test::Harness::runtests(@tests);
   } else {
     print("No tests defined.\n");
@@ -447,38 +502,57 @@ sub ACTION_test {
   # This will get run and the user will see the output.  It doesn't
   # emit Test::Harness-style output.
   if (-e 'visual.pl') {
-    $self->run_script('visual.pl', '-Mblib');
+    $self->run_perl_script('visual.pl', '-Mblib');
   }
 }
 
 sub ACTION_testdb {
   my ($self) = @_;
-  local $self->{args}{debugger} = 1;
+  local $self->{properties}{debugger} = 1;
   $self->depends_on('test');
 }
 
 sub ACTION_build {
   my ($self) = @_;
   
-  if ($self->{args}{c_source}) {
-    push @{$self->{include_dirs}}, $self->{args}{c_source};
-    my $files = $self->rscan_dir($self->{args}{c_source}, qr{\.(c|PL)$});
+  if ($self->{properties}{c_source}) {
+    $self->process_PL_files($self->{properties}{c_source});
+    
+    my $files = $self->rscan_dir($self->{properties}{c_source}, qr{\.c$});
+    
+    push @{$self->{include_dirs}}, $self->{properties}{c_source};
+
     foreach my $file (@$files) {
-      if ($file =~ /c$/) {
-	push @{$self->{objects}}, $self->compile_c($file);
-      } elsif ($file =~ /PL/) {
-	$self->run_perl_script($file);
-      }
+      push @{$self->{objects}}, $self->compile_c($file);
     }
   }
 
   # What more needs to be done when creating blib/ from lib/?
   # Currently we handle .pm, .xs, .pod, and .PL files.
-  my $files = $self->rscan_dir('lib', qr{\.(pm|pod|xs|PL)$});
+
+  $self->process_PL_files('lib');
+
+  my $files = $self->rscan_dir('lib', qr{\.(pm|pod|xs)$});
   $self->lib_to_blib($files, 'blib');
   $self->add_to_cleanup('blib');
 }
 
+sub process_PL_files {
+  my ($self, $dir) = @_;
+  my $p = $self->{properties}{PL_files};
+  my $files = $self->rscan_dir($dir, qr{\.PL$});
+  foreach my $file (@$files) {
+    my @to = (exists $p->{$file} ?
+	      (ref $p->{$file} ? @{$p->{$file}} : ($p->{$file})) :
+	      $file =~ /^(.*)\.PL$/);
+    
+    if (grep {!-e $_ or  -M _ > -M $file} @to) {
+      $self->run_perl_script($file);
+      $self->add_to_cleanup(@to);
+    }
+  }
+}
+
 sub ACTION_install {
   my ($self) = @_;
   require ExtUtils::Install;
@@ -503,7 +577,7 @@ sub ACTION_clean {
 sub ACTION_realclean {
   my ($self) = @_;
   $self->depends_on('clean');
-  $self->delete_filetree($self->{config_dir}, $self->{build_script});
+  $self->delete_filetree($self->{properties}{config_dir}, $self->{properties}{build_script});
 }
 
 sub ACTION_dist {
@@ -577,8 +651,8 @@ sub ACTION_manifest {
 sub dist_dir {
   my ($self) = @_;
 
-  (my $dist_dir = $self->{args}{module_name}) =~ s/::/-/;
-  return "$dist_dir-$self->{args}{module_version}";
+  (my $dist_dir = $self->{properties}{module_name}) =~ s/::/-/;
+  return "$dist_dir-$self->{properties}{module_version}";
 }
 
 sub make_tarball {
@@ -594,8 +668,8 @@ sub install_map {
   my ($self, $blib) = @_;
   my $lib  = File::Spec->catfile($blib,'lib');
   my $arch = File::Spec->catfile($blib,'arch');
-  return {$lib  => $self->{args}{sitelib},
-	  $arch => $self->{args}{sitearch},
+  return {$lib  => $self->{config}{sitelib},
+	  $arch => $self->{config}{sitearch},
 	  read  => ''};  # To keep ExtUtils::Install quiet
 }
 
@@ -636,8 +710,8 @@ sub lib_to_blib {
   # Create $to/arch to keep blib.pm happy (what a load of hooie!)
   File::Path::mkpath( File::Spec->catdir($to, 'arch') );
 
-  if ($self->{args}{autosplit}) {
-    $self->autosplit_file($self->{args}{autosplit}, $to);
+  if ($self->{properties}{autosplit}) {
+    $self->autosplit_file($self->{properties}{autosplit}, $to);
   }
   
   foreach my $file (@$files) {
@@ -648,9 +722,6 @@ sub lib_to_blib {
     } elsif ($file =~ /\.xs$/) {
       $self->process_xs($file);
 
-    } elsif ($file =~ /\.PL$/) {
-      $self->run_perl_script($file);
-
     } else {
       warn "Ignoring file '$file', unknown extension\n";
     }
@@ -667,18 +738,18 @@ sub autosplit_file {
 
 sub compile_c {
   my ($self, $file) = @_;
-  my $args = $self->{args}; # For convenience
+  my $cf = $self->{config}; # For convenience
 
   # File name, minus the suffix
   (my $file_base = $file) =~ s/\.[^.]+$//;
-  my $obj_file = "$file_base$args->{obj_ext}";
+  my $obj_file = "$file_base$cf->{obj_ext}";
   return $obj_file if $self->up_to_date($file, $obj_file);
   
   $self->add_to_cleanup($obj_file);
-  my $coredir = File::Spec->catdir($args->{archlib}, 'CORE');
+  my $coredir = File::Spec->catdir($cf->{archlib}, 'CORE');
   my $include_dirs = $self->{include_dirs} ? join ' ', map {"-I$_"} @{$self->{include_dirs}} : '';
-  $self->do_system("$args->{cc} $include_dirs -c $args->{ccflags} -I$coredir -o $obj_file $file")
-    or die "error building $args->{dlext} file from '$file'";
+  $self->do_system("$cf->{cc} $include_dirs -c $cf->{ccflags} -I$coredir -o $obj_file $file")
+    or die "error building $cf->{dlext} file from '$file'";
 
   return $obj_file;
 }
@@ -686,14 +757,14 @@ sub compile_c {
 sub run_perl_script {
   my ($self, $script, $preargs, $postargs) = @_;
   $preargs ||= '';   $postargs ||= '';
-  return $self->do_system("$self->{args}{perl5} $preargs $script $postargs");
+  return $self->do_system("$self->{config}{perlpath} $preargs $script $postargs");
 }
 
 # A lot of this looks Unixy, but actually it may work fine on Windows.
 # I'll see what people tell me about their results.
 sub process_xs {
   my ($self, $file) = @_;
-  my $args = $self->{args}; # For convenience
+  my $cf = $self->{config}; # For convenience
 
   # File name, minus the suffix
   (my $file_base = $file) =~ s/\.[^.]+$//;
@@ -707,7 +778,7 @@ sub process_xs {
     my $typemap =  $self->module_name_to_file('ExtUtils::typemap');
     
     # XXX the '> $file_base.c' isn't really a post-arg, it's redirection.  Fix later.
-    $self->run_perl_script($xsubpp, "-I$args->{archlib} -I$args->{privlib}", 
+    $self->run_perl_script($xsubpp, "-I$cf->{archlib} -I$cf->{privlib}", 
 			   "-noprototypes -typemap '$typemap' $file > $file_base.c")
       or die "error building .c file from '$file'";
   }
@@ -734,13 +805,13 @@ sub process_xs {
   $self->copy_if_modified("$file_base.bs", $archdir, 1);
   
   # .o -> .(a|bundle)
-  my $lib_file = File::Spec->catfile($archdir, File::Basename::basename("$file_base.$args->{dlext}"));
-  unless ($self->up_to_date("$file_base$args->{obj_ext}", $lib_file)) {
-    my $linker_flags = $args->{extra_linker_flags} || '';
+  my $lib_file = File::Spec->catfile($archdir, File::Basename::basename("$file_base.$cf->{dlext}"));
+  unless ($self->up_to_date("$file_base$cf->{obj_ext}", $lib_file)) {
+    my $linker_flags = $cf->{extra_linker_flags} || '';
     my $objects = $self->{objects} || [];
-    $self->do_system("$args->{shrpenv} $args->{cc} $args->{lddlflags} -o $lib_file ".
-		     "$file_base$args->{obj_ext} @$objects $linker_flags")
-      or die "error building $args->{obj_ext} file from '$file_base.$args->{dlext}'";
+    $self->do_system("$cf->{shrpenv} $cf->{cc} $cf->{lddlflags} -o $lib_file ".
+		     "$file_base$cf->{obj_ext} @$objects $linker_flags")
+      or die "error building $file_base$cf->{obj_ext} from '$file_base.$cf->{dlext}'";
   }
 }
 
